--  Telemetry script: RES Training App
--  #################################
--
--  version 16   10.05.2022
--  by ChrisOHara
--  original written on OpenTX version 2.3.7; last version on 2.3.15
--  for Horus and Taranis radios (but may run on other openTX radios)
--  This Software is free
--
--  Information: Using a barometer this software is not allowed in official RES competion.
--
--  Requirements: Telemetry value "Alt" (so a barometer is required)
--                Momentary switch has to be definded to match logical switch ls61
--                   (example for ls61:   L61 OR SEv  SEv )
-- 
--
--  Reaching 5m (starting height) you will hear a beep (1000 Hz).
--  During the following 15 seconds you can start the timer by operating momentary switch "ls61"
--     when plane is released from leash.
--  If you do not operate a switch  the  timer will start after 15 seconds on its own.
--  When timer starts the current altitude is announced.
--
--  reaching landing height (2m) there is another beep (1000Hz).
--  During the next 15 seconds you can stop the timer by operating momentary switch "ls61"
--     when having contact with ground.
--  If you do not operate a switch the timer will stop after 15 seconds on its own.
--  When timer stops you can hear a beep (2000Hz).
--
--  Hint: If you do not use a switch  you will get the flighttime between 
--        5m (starting altidude) and 2m (landing altidude).
--
--  The script does use/define timer 1 on its own. (6 minutes, countdown)
--
--  The last flights will be displayed. 
--      Horus: max is 2,6,8,19 depending on screen zone you select for widget
--      Taranis: max is 8
--
--  Information displayed:
--      Flight     =    Start-time of flight hour:minutes (newest flight always on top)
--      Time       =    Flight-time in minutes:seconds
--      Alt-S      =    starting altidude
--      Alt-F      =    maximum altidude (during flight from start timer to stop timer)
--
--  Flight status:
--      Global variable Gvar9 FM0 does show the status of the flight
--
--
--  New features in version 16
--    1. the script writes a logfile to sdcard folder /LOGS/
--       name of the file is: ResLog--MODELNAME.csv
--       logfile is in csv format and can be opened with Excel
--	 When script is activated it reads the logfile of the model and displays the last flights
--
--    2. the definition of the switch to be used is done outside the script  by defining a momentary switch to ls61
--
--    3. When timer is started beep is in again and start altitude is announced after beep.
--
--    4. The script does set a value to Gvar9 (Global Variable 9) of FM0 (Flight Mode 0) which can be used for logical switches.
--          0   = waiting for receiver to be armed
--          1   = waiting 3 seconds to reset telemetry values "Alt" and VSped"
--          2   = Waiting for launch (less than 5m altitude)
--          3   = launching 5m altitude is reached but timer has not been started.
--          4   = released from leach (flying), timer is running
--          5   = landing, altitude is below 2m but timer is still running
--       example: I do use this with a logical switch so that my taranis does play-value "Alt" during condition 4 only.
--
--       if you want to use another global variable or want to dismiss it look for this lines and follow instructions
--
--         -- you can store the flight_condition in a in a global variable of a flightmode to use them for logical switches or special functions
--         -- you can choose the gvar and flight_number (or dismiss storage by setting one incorrect value)
--         local my_gvar_index = 8;             -- (0 to 8)   select another number (for example -1 or 9) if you do not want to store the condition in a global variable)
--         local my_flightmode_index = 0;       -- (0 to 8)   select another number (for example -1 or 9) if you do not want to store the condition in a global variable)
--
--  New features in Version 13:
--    1. When timer is started the current altitude is anounced (instead of beep).
--    2. The script does work also with unit "ft", when unit "ft" is set for telemetry value "Alt".
--       Starting-/landing-altitude (5m and 2m) are automatically recalculated in "ft".
--    3. Time slot for starting and stopping timer is now 15 seconds (was 10).
--    4. While flying there is no realistic chance to read the displayed list.
--       So the display now shows altitude and time as large as possible instead (only while flight is going on)
--    6. The file dummy.lua in /SCRIPTS/TELEMETRY/ is no longer needed.
--    7. Did some optimization concerning CPU usage in the Horus version.
--
--  New features in Version 11:
--    1. If receiver is switch off you get a 3s beep and a warning on the screen "not armed".
--       If you don't like that you search the two lines starting with "-- armed-warning" at the end
--         and change the line to comment.
--       (I added this function after I successfully launching my plane without switching on the receiver. Haha)
--    2. The GR6 firmware (at least access version) has a firmware bug. When switching on the receiver the altitdue rises
--        from zero to geografical altitude. Therefore openTX function auto-offset does not work properly and also this
--        lua script wrongly started a flight. Now the script does reset the telemetry value "Alt" (and "VSpd") three seconds 
--        after the receiver is switched on. But this works only for OpenTx 2.3.11 or higher.
--        For older releases there is only one solution: Do switch on the receiver first and then switch on the transmitter.
--        Or select the model after switching on the receiver.
--    3. In the Horus version the code was moved from the telemetry script to the widget (telemetry script is a dummy now)
--
--  Installation:
--      Horus: Copy /WIDGETS/res_H/main.lua to the sd_card of your Horus.
--             Add the widget res_H to your telemetry screen.
--      Taranis: Copy /SCRIPTS/TELEMETRY/res_T.lua   to the sd_card of your Taranis.
--               Add the script res_T to your telemetry screen.
--
--  Errors: Scripts aborts when function getValue() is called with nil.
--           -> Telemetry value "Alt" is not available 
--           -> or logical switch LS61 is not defined
--
-- 
--  #######################################################################################################



local default_start_altitude = 5;     -- the altitude to recognize starting condition in [m] (script will recalculate if Alt-sensor unit=feet)
local default_landing_altitude = 2;   -- the altitude to recognize landing condition in [m] (script will recalculate if Alt-sensor unit=feet)
local start_stop_time_delta = 15;     -- maximum time-delta to press timer button
local my_timer_number = 0;	      -- the timer that is resetted by start condition

--   you can store the flight_condition in a in a global variable of a flightmode to use them for logical switches or special functions
--   you can choose the gvar and flight_number (or dismiss storage by setting one incorrect value)
local my_gvar_index = 8;             -- (0 to 8)   select another number (for example -1 or 9) if you do not want to store the condition in a global variable)
local my_flightmode_index = 0;       -- (0 to 8)   select another number (for example -1 or 9) if you do not want to store the condition in a global variable)

local my_gvar_store = 0;          

local switch_condition_start = -1024;
local switch_condition_stop = -1024;
local ls61_ID;				 --  defines switch  as switch for time meassurement
local my_timer = nil;
local start_time;
local flight_time_start_end_limit;
local altitude = 0;
local max_altitude = 0;
local altId = nil;
local my_condition = 0;          -- 0="looking for armed"; 1="looking for telemetry restet", 2="looking for start"
                                 -- 3="looking for flying"; 4="looking for landing"; 5="looking for stop timer"

local max_flight_counter = 8;    -- maximum number of flights stored, prevents screen overflow
local flight_count = 0;          -- counts flights stored
local i=0;
local my_minutes = 0;
local my_seconds = 0;
local flight_time_0 = 0;
local flight_start_times = {};
local flight_times = {};
local start_altitudes = {};
local flight_altitudes = {};
local my_Alt_unit = UNIT_METERS;
local my_start_altitude = 0;
local my_landing_altitude = 0;

local my_armed = 100;
local my_sensor_reset_time = 0;

local my_logfilename="";
local my_date="";

-- ############################# init function ##################
local function init_func()
    altId = getFieldInfo("Alt") and getFieldInfo("Alt").id or nil;
    ls61_ID = getFieldInfo("ls61") and getFieldInfo("ls61").id or nil ;
    my_timer = model.getTimer(my_timer_number);
      my_timer.mode = 0;
      my_timer.start = 360;
      my_timer.value = 360;
      my_timer.countdownBeep = 2;
      my_timer.minuteBeep = false;
      my_timer.persistant = 0;
    model.setTimer(my_timer_number , my_timer);

    --    ###### reading old data from Logfiles and 

    local my_model = model.getInfo();
    my_logfilename="/LOGS/ResLog--" .. my_model.name .. ".csv";
    local xxx=getDateTime();
    my_date=xxx.year.."."..xxx.mon.."."..xxx.day;
    
    local my_file = io.open (my_logfilename,"r");
    local my_line="";


    if my_file == nil then                        -- no logfile found, generate new logfile

	  my_file = io.open (my_logfilename,"a");
  	  io.write(my_file, "Date;Flight;Time;Alt-S;Alt-F;#\n");

        else
   
          local my_pos=0;
	  local my_endpos=0;
	  io.seek(my_file, my_pos); my_line = io.read(my_file,100); my_endpos = string.find(my_line, "\n");		-- read header line

	  while my_endpos ~= nil do
	   for k=1,1,1 do           -- for-dummy loop to enable use break without leaving while-loop
	     my_pos=my_pos+my_endpos; 
	     io.seek(my_file, my_pos); my_line = io.read(my_file,100); my_endpos = string.find(my_line, "\n"); 		-- read data line
             local x1 = string.find(my_line,";");
	      if x1 == nil then break end;
              x1=x1+1;
              local x2 = string.find(my_line,";",x1);
	      if x2 == nil then break end;
	      x2=x2-1;
	     local x_flight_start_times = string.sub (my_line, x1, x2);
	     if x_flight_start_times == nil then break end;
	      x1 = x2 + 2;
              x2 = string.find(my_line,";",x1);
	      if x2 == nil then break end;
	      x2=x2-1;
	     local x_flight_times = string.sub (my_line, x1, x2);
	     if x_flight_times == nil then break end;
	      x1 = x2 + 2;
              x2 = string.find(my_line,";",x1);
	      if x2 == nil then break end;
	      x2=x2-1;
	     local x_start_altitudes = tonumber(string.sub (my_line, x1, x2));
	     if x_start_altitudes == nil then break end;
	      x1 = x2 + 2;
              x2 = string.find(my_line,";#",x1);
	      if x2 == nil then break end;
	      x2=x2-1;
             local x_flight_altitudes = tonumber(string.sub (my_line, x1, x2));
	     if x_flight_altitudes == nil then break end;

             if flight_count < max_flight_counter then flight_count = flight_count + 1; end;		-- new data found

	     for i = (max_flight_counter-1),1,-1 do  				  -- shift Globalvariables to empty first slot for newer values
        	flight_start_times [i] = flight_start_times [i-1];
                flight_times [i] = flight_times [i-1];
                start_altitudes [i] = start_altitudes [i-1];
                flight_altitudes [i] = flight_altitudes [i-1];
               end -- end for

 	     flight_start_times [0] = x_flight_start_times;
             flight_times [0] = x_flight_times;
             start_altitudes [0] = x_start_altitudes;
	     flight_altitudes [0] = x_flight_altitudes;
	end -- end for-dummy loop
       end -- end while
    end -- end if my-file

    io.close(my_file);

    --   ###### end of reading old data from logfile

end


-- ############################ store condition in Gvar ########################################
local function store_gvar()

    if      my_gvar_store == 1 then 
        model.setGlobalVariable(my_gvar_index, my_flightmode_index, my_condition);
    elseif  my_gvar_store == -1 then
        return;
    elseif  my_gvar_store == 0 then  
        if( (my_gvar_index < 0) or (my_gvar_index > 8) or (my_flightmode_index < 0) or (my_flightmode_index > 8) ) then 
            my_gvar_store = -1;
          else
            my_gvar_store =  1;
          end
    end

end


-- ############################# looking for armed ##################
local function look_for_armed()           -- called when condition is 0
  if my_armed < 2 then 
       playTone(800, 3000, 6000, PLAY_NOW ) ;
     else
       my_sensor_reset_time = getTime() + 300;       -- delta time to reset is 300*0.01 = 3 seconds
       my_condition = 1;
       store_gvar();
     end
end

-- ############################ lookig for reset of sensor ###############################
local function look_for_reset_sensor()     

   if getTime() < my_sensor_reset_time then                -- called as long as my_condition = 1
      return;
      end

   my_condition = 2;
   store_gvar();

   local ver, radio, maj, minor, rev = getVersion()       -- finding version of OTx
   local my_version =maj*10000 + minor*100 + rev ;


    for i = 0, 31 do                           -- finding sensorID Alt and VSpd and reset, switch from Meter to Feet
      local my_sensor = model.getSensor(i);
      if my_sensor.name == "Alt" then
          if my_version >= 20311 then model.resetSensor(i) end; -- does need openTx 2.3.11 or higher
          my_Alt_unit = my_sensor.unit ;
          if my_Alt_unit == UNIT_FEET then
	      my_start_altitude   = default_start_altitude * 3,28084;
              my_landing_altitude = default_landing_altitude * 3,28084;
            else
	      my_start_altitude   = default_start_altitude;
              my_landing_altitude = default_landing_altitude;             
            end
        end
      if my_sensor.name == "VSpd" then
          if my_version >= 20311 then model.resetSensor(i) end; -- does need openTx 2.3.11 or higher
        end
      end

end
     
  

-- #############################  looking for start #############
local function look_for_start()                                     -- called as long as my_condition is 2
   altitude=getValue(altId);
   if altitude > my_start_altitude then 
     max_altitude = altitude;
     for i = (max_flight_counter-1),1,-1 do    -- shift Globalvariables to empty first slot for newer values
        flight_start_times [i] = flight_start_times [i-1];
        flight_times [i] = flight_times [i-1];
        start_altitudes [i] = start_altitudes [i-1];
        flight_altitudes [i] = flight_altitudes [i-1];
       end
     max_altitude = 0;
     start_altitudes[0]=0;
     flight_start_times[0] = "00:00";
     flight_times[0] = "0:00";
     flight_altitudes[0] = 0;
     if flight_count < max_flight_counter then
        flight_count = flight_count + 1;
       end
     flight_time_start_end_limit = getTime() + start_stop_time_delta*100;
     switch_condition_start = -1024; 
     switch_condition_stop  = -1024;
     playTone(1000, 500, 500, PLAY_NOW); 
     my_condition = 3;
     store_gvar();
   end
end

-- #############################  looking for flying (release from leach) ############
local function look_for_flying()                                   -- called as long as my_condition is 3
   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        start_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
	flight_altitudes[0] = start_altitudes[0];
     end

   start_time=getTime();

   if ((getValue(ls61_ID) > 1000)) then
	switch_condition_start=1024;
     end
   if ((start_time >= flight_time_start_end_limit) or (switch_condition_start == 1024) ) then
        local my_date_time = getDateTime();
	flight_start_times[0] = string.format("%02d:%02d", my_date_time.hour, my_date_time.min);
        my_timer.mode = 1;
        model.setTimer(my_timer_number , my_timer );
	playTone(2000, 500, 500, PLAY_NOW);
        playNumber(start_altitudes[0]*10, my_Alt_unit, PREC1 );
	my_condition = 4;
        store_gvar();
     end

end

-- #############################  looking for landing ############
local function look_for_landing()                                   -- called as long as my_condition is 4
   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        flight_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
     end

   flight_time_0 = math.floor((getTime() - start_time) / 100 + 0.5);
   my_minutes = math.floor(flight_time_0 / 60);
   my_seconds = flight_time_0 - (my_minutes * 60);
   flight_times[0] = string.format("%d:%02d", my_minutes, my_seconds);

   if altitude < my_landing_altitude then
        flight_time_start_end_limit = flight_time_0 + start_stop_time_delta;
	playTone(1000, 500, 500, PLAY_NOW);
	my_condition = 5;
        store_gvar();
     end
end

-- ############################# looking for stop timer ###########
local function look_for_stop_timer()                                 -- called as log as my_condition is 5

   flight_time_0 = math.floor((getTime() - start_time) / 100 + 0.5);
   my_minutes = math.floor(flight_time_0 / 60);
   my_seconds = flight_time_0 - (my_minutes * 60);
   flight_times[0] = string.format("%d:%02d", my_minutes, my_seconds);

   if ( (getValue(ls61_ID) > 1000)) then
	switch_condition_stop=1024; 
     end
   if ((flight_time_0 >= flight_time_start_end_limit) or (switch_condition_stop == 1024) ) then
        my_timer.mode = 0;
        model.setTimer(my_timer_number , my_timer);
	playTone(2000, 500, 500, PLAY_NOW);
 	my_condition = 2;
        store_gvar();

     --  #### writing new data to logfile
        local my_file = io.open (my_logfilename,"a");
          io.write(my_file, my_date..";"..flight_start_times[0]..";"..flight_times[0]..";"..start_altitudes[0]..";"..flight_altitudes[0]..";#\n");
          io.close(my_file);
     --  #### end of writing new data to logfile

    end

end


-- ############################### background function #################
local function bg_func()
  my_armed = getRSSI() ;
  if my_armed < 2 then 
      my_condition = 0;
      store_gvar();
      end;
  if       my_condition == 0 then look_for_armed();  
    elseif my_condition == 1 then look_for_reset_sensor();
    elseif my_condition == 2 then look_for_start();
    elseif my_condition == 3 then look_for_flying();
    elseif my_condition == 4 then look_for_landing();
    elseif my_condition == 5 then look_for_stop_timer();
    end
end

-- ############################### run function #################
local function run_func()
    lcd.clear()

    if my_condition <= 2 then
          lcd.drawText (5,0,"Flight  Time   Alt-S   Alt-F", SMLSIZE+INVERS);
          for i=0,(flight_count-1) do
	      local yy = 8+i*7;
	      lcd.drawText (5,yy,flight_start_times[i], SMLSIZE);
 	      lcd.drawText (40,yy,flight_times[i], SMLSIZE);
	      lcd.drawText (67,yy,start_altitudes[i], SMLSIZE);
	      lcd.drawText (100,yy,flight_altitudes[i], SMLSIZE);
            end
          if my_condition == 0 then  lcd.drawText (30,25,"not armed", MIDSIZE); end
      else
         lcd.drawText (0,  0, math.floor(altitude*10)/10 , XXLSIZE);
         lcd.drawText (0, 40, flight_times[0], DBLSIZE);
    end

end


return { run=run_func, background=bg_func, init=init_func }
